#!/usr/bin/perl -w -I .
#-I /usr/local/perl

=pod

 Copyright Heiner Kuhlmann : hkuhlmann@dr-k.de Praktische-Elektronik.dr-k.de
 The software is licensed under the GNU Library General Public License.

=head1 NAME:

LogicBuddy - LogicBuddy GUI

=head1 SYNOPSIS:

LogicBuddy

=head1 ABSTRACT:

LogicBuddy supports solving logical problems.

=head1 DESCRIPTION:

LogicBuddy supports

   creating truth tables

   minimizing logical expressions

=head2 OPTIONS:

   none

head1 AUTHOR:

Heiner Kuhlmann, E<lt>hkuhlmann@dr-k.deE<gt>

=head1 COPYRIGHT AND LICENSE:

Copyright Heiner Kuhlmann : hkuhlmann@dr-k.de Parktische-Elektronik.dr-k.de
The software is licensed under the GNU Library General Public License.

=cut

use strict;
use warnings;

BEGIN {
   use File::Basename;
   my $local_directory = File::Basename::dirname( File::Basename::dirname( $0 ) );
   push @INC, $local_directory;
};

use Readonly;
eval "use Win32::GUI;";
eval { 
       my ( $DOS ) = Win32::GUI::GetPerlWindow();
       Readonly $LogicBuddy::C_DOS_window => $DOS;
       Win32::GUI::Hide($DOS);
     };

use Cwd;
use File::Basename;

use LogicBuddy::Tk::Configuration;

use Tk;
use Tk::Help;
use Tk::Images;

my $start = File::Basename::dirname( $0 );
PATH: foreach my $root ( $start, @INC, Cwd::cwd(), File::HomeDir->my_home() ) {
   if ( $root and -e "$root/LogicBuddy_Main.png" ) {
      Tk::Images::add_path( $root );
      last PATH;
   } elsif ( $root and -e "$root/images/LogicBuddy_Main.png" ) {
      Tk::Images::add_path( "$root/images" );
      last PATH;
   } elsif ( $root and -e "$root/LogicBuddy/images/LogicBuddy_Main.png" ) {
      Tk::Images::add_path( "$root/LogicBuddy/images" );
      last PATH;
   };
}; # PATH

use LogicBuddy::Configuration;
use LogicBuddy::Locale;

use LogicBuddy::Tk::Viewer;

use constant FALSE => 0;
use constant TRUE  => 1;

local $SIG{__WARN__} = \&Carp::confess;
local $SIG{__DIE__}  = \&Carp::confess;

$LogicBuddy::Tk::G_top = Tk::MainWindow->new;
$LogicBuddy::Tk::G_top->iconimage( Tk::Images::LogicBuddy($LogicBuddy::Tk::G_top) ) if Tk::Images::LogicBuddy( $LogicBuddy::Tk::G_top );
$LogicBuddy::Tk::G_top->bind( '<Key-F1>' => sub { LogicBuddy::Tk::Help::main_help( $LogicBuddy::Tk::G_top ); } );
$LogicBuddy::Tk::G_top->bind( '<Button-3>' => sub { LogicBuddy::Tk::Help::main_help( $LogicBuddy::Tk::G_top ); } );
my ( $min_width, $min_height ) = @Tk::Configuration::C_standard_minsize;
my $screen_width  = List::Util::min( int( 0.8 * $LogicBuddy::Tk::G_top->screenwidth() ),  $min_width );
my $screen_height = List::Util::min( int( 0.8 * $LogicBuddy::Tk::G_top->screenheight() ), $min_height );

$LogicBuddy::Tk::G_top->minsize( $screen_width, $screen_height );

my $viewer  = new LogicBuddy::Tk::Viewer( $LogicBuddy::Tk::G_top );
my $buttons = $viewer->control_buttons( $LogicBuddy::Tk::G_top );

Tk::Configuration::set_options( $LogicBuddy::Tk::G_top );

$buttons->pack( -fill => 'x' );
$viewer->pack( -fill => 'both', -expand => TRUE );
$LogicBuddy::Tk::G_top->update();

Tk::MainLoop;

eval { Win32::GUI::Show($LogicBuddy::C_DOS_window) if $LogicBuddy::C_DOS_window; };

