=pod

Copyright Heiner Kuhlmann : hkuhlmann@dr-k.de Praktische-Elektronik.dr-k.de
The software is licensed under the GNU Library General Public License.

=head1 NAME

LogicBuddy::Tk::Configuration - module

=head1 SYNOPSIS

use LogicBuddy::Tk::Configuration;

=head1 ABSTRACT

LogicBuddy::Tk::Configuration provides LogicBuddy Tk configuration.

They are defined as Tk::Configuration::C_ ...

All configurations are Readonly.

LogicBuddy::Tk::Configuration use the configurations of Tk::StandardConfiguration.

Some configurations of Tk::Configuration are changed using a BEGIN block.

Other configurations are defined here.

=head1 DESCRIPTION

=head2 SUBPROGRAMS

   No SUBPROGRAMS are defined.

=head1 AUTHOR

Heiner Kuhlmann, E<lt>hkuhlmann@dr-k.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright Heiner Kuhlmann : hkuhlmann@dr-k.de Praktische-Elektronik.dr-k.de
The software is licensed under the GNU Library General Public License.

=cut

use strict;

################################################################################
# internal subprograms
################################################################################

package LogicBuddy::Tk::Configuration;

################################################################################
# import
################################################################################

################################################################################
# global constants
################################################################################

use constant FALSE => 0;
use constant TRUE  => 1;

################################################################################
# LogicBuddy Tk global configuration
################################################################################

################################################################################
# global variable
################################################################################

use Readonly;

BEGIN {
   eval { Readonly $Tk::Configuration::C_fixed_font_size => ( $^O eq 'MSWin32' )  ?   13    : 11 };
   eval { Readonly $Tk::Configuration::C_reserved_color  => 'mediumblue' };
   eval { Readonly $Tk::Configuration::C_input_color     => 'forestgreen' };
   eval { Readonly $Tk::Configuration::C_output_color    => 'firebrick' };
   eval { Readonly $Tk::Configuration::C_comment_color   => 'slategrey' };
   eval { Readonly $Tk::Configuration::C_min_width       => 250 };
   eval { Readonly $Tk::Configuration::C_min_height      => 125 };
   eval { Readonly $Tk::Configuration::C_standard_height => 400 };
   eval { Readonly $Tk::Configuration::C_standard_width  => ( $^O eq 'MSWin32' )  ?   770   :   650 };

}; # BEGIN

use Tk::StandardConfiguration;

# can be predefined : eval

eval { Readonly $Tk::Configuration::C_margin             => '   ' };

eval { Readonly $Tk::Configuration::C_message_height     =>   8 }; # lines
eval { Readonly $Tk::Configuration::C_editor_height      =>   8 }; # lines
eval { Readonly $Tk::Configuration::C_text_height        =>  16 }; # lines
eval { Readonly $Tk::Configuration::C_text_width         =>  64 }; # character
eval { Readonly $Tk::Configuration::C_editor_min_height  => 400 }; # pixel
eval { Readonly $Tk::Configuration::C_editor_min_width   => 650 }; # pixel

eval { Readonly @Tk::Configuration::C_editor_minsize     => ( $Tk::Configuration::C_editor_min_width, $Tk::Configuration::C_editor_min_height ); };
eval {
       Readonly %Tk::Configuration::C_Text_configuration => ( -width      => $Tk::Configuration::C_text_width,
                                                              -height     => $Tk::Configuration::C_text_height,
                                                              -wrap       => 'word',
                                                              -background => $Tk::Configuration::C_background,
                                                              -font       => $Tk::Configuration::C_bold_fixed_font,
                                                              %Tk::Configuration::C_Standard_Pads,
                                                            );
     };
eval { Readonly @Tk::Configuration::C_standard_minsize   => ( $Tk::Configuration::C_standard_width, $Tk::Configuration::C_standard_height ); };
eval { Readonly @Tk::Configuration::C_dialog_minsize     => ( $Tk::Configuration::C_min_width, $Tk::Configuration::C_min_height ); };

################################################################################
# internal subprograms
################################################################################

################################################################################
# internal Tk subprograms
################################################################################

################################################################################
# internal call back subprograms
################################################################################

################################################################################
# exported subprograms
################################################################################

################################################################################
# end LogicBuddy::Tk::Configuration
################################################################################

1;
