=pod

Copyright Heiner Kuhlmann : hkuhlmann@dr-k.de Praktische-Elektronik.dr-k.de
The software is licensed under the GNU Library General Public License.

=head1 NAME

LogicBuddy::Tk::Help - module

=head1 SYNOPSIS

use LogicBuddy::Tk::Help;

$map = LogicBuddy::Tk::Help::main_help( $parent )

$map = LogicBuddy::Tk::Help::help_license( $a_map );

$map = LogicBuddy::Tk::Help::help_artistic_license( $a_map );

$map = LogicBuddy::Tk::Help::help_gnu_lesser_license( $a_map );

$map = LogicBuddy::Tk::Help::help_espresso( $a_map );

$map = LogicBuddy::Tk::Help::help_strawberry_perl( $a_map );

$map = LogicBuddy::Tk::Help::help_dialog( $a_map );

$map = LogicBuddy::Tk::Help::help_last_solution_window( $a_map );

$map = LogicBuddy::Tk::Help::help_truth_table_window( $a_map );

$map = LogicBuddy::Tk::Help::help_change_variable_name( $a_map );

$map = LogicBuddy::Tk::Help::help_select_logic( $a_map );

$map = LogicBuddy::Tk::Help::help_define_new_table( $a_map );

$map = LogicBuddy::Tk::Help::help_change_multiple( $a_map );

$map = LogicBuddy::Tk::Help::messages_help( $a_map );

$map = LogicBuddy::Tk::Help::editor_help( $a_map );

$map = LogicBuddy::Tk::Help::help_status( $a_map );

$map = LogicBuddy::Tk::Help::help_editor_window( $a_map );

$map = LogicBuddy::Tk::Help::help_syntax( $a_map );

$map = LogicBuddy::Tk::Help::help_full_syntax( $a_map );

$map = LogicBuddy::Tk::Help::help_reserved_words( $a_map );

$map = LogicBuddy::Tk::Help::help_syntax_rules( $a_map );

$map = LogicBuddy::Tk::Help::help_syntax_example( $a_map );

$map = LogicBuddy::Tk::Help::help_editor_error_window( $a_map );

=head1 ABSTRACT

LogicBuddy::Tk::Viewer abstracts Help support for LogicBuddy GUI.

=head1 DESCRIPTION

=head2 SUBPROGRAMS

=head3 main_help

   $map = LogicBuddy::Tk::Help::main_help( $parent )

Opens the main help dialog. The dialog uses the $parent.

The $map is the dialog of main_help.
The $map should be used to open a help sub dialog.

=head3 help sub dialogs

   $map = LogicBuddy::Tk::Help::Help_dialog( $a_map );

Help_dialog is an example of a help sub dialog.

The help_dialog is opened referring to $a_map.
The $map is the dialog of help_dialog.
The $map should be used to open a sub dialog of the help_dialog.

=head1 AUTHOR

Heiner Kuhlmann, E<lt>hkuhlmann@dr-k.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright Heiner Kuhlmann : hkuhlmann@dr-k.de Praktische-Elektronik.dr-k.de
The software is licensed under the GNU Library General Public License.

=cut

use strict;

use LogicBuddy::Configuration;
use LogicBuddy::Plural;
use LogicBuddy::Singular;
use LogicBuddy::Logic::Text;

################################################################################
## LogicBuddy::Tk::Help
#################################################################################

package LogicBuddy::Tk::Help;

use English '-no_match_vars';

################################################################################
# global constants
################################################################################

use constant FALSE => 0;
use constant TRUE  => 1;
use constant EMPTY => '';

use Readonly;

Readonly my $C_artistic         => 'artistic.txt';
Readonly my $C_GNU              => 'lgpl-3.0.txt';

Readonly my $C_primary_license  => LogicBuddy::Singular::Copyright_HK( ".\n\n" ) .
                                   LogicBuddy::Singular::This_is_free_software_licensed_under( ":\n" ) .
                                   "o   The Artistic License 2.0\n" .
                                   "o   GNU LESSER GENERAL PUBLIC LICENSE Version 3";
Readonly my $C_artistic_missing => "The $C_artistic file is missing. It has to be included in this software Package. " .
                                   "Please read $C_artistic.\n\n$C_primary_license";
Readonly my $C_GNU_missing      => "The $C_GNU file is missing. It has to be included in this software Package. " .
                                   "Please read $C_GNU.\n\n$C_primary_license";

################################################################################
# global variable
################################################################################

################################################################################
# internal subprograms
################################################################################

sub I_insert_state_icons {
   my ( $dialog ) = @_;

   Tk::Help::insert_help( $dialog, EMPTY,
      [ 'bold', LogicBuddy::Singular::Status( ': ') . 
        LogicBuddy::Singular::match_between_logical_function_and_table()
      ]
   );

   my %defined_states = LogicBuddy::Tk::Editor::defined_states();
   foreach my $status ( sort (keys %defined_states) ) {
      Tk::Help::insert_help( $dialog, [ 'image', $defined_states{$status}[1] ], $defined_states{$status}[0] );
   };

}; # I_insert_state_icons

sub I_describe_result_function {
   my ( $dialog ) = @_;

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::There_are_various_types_representing_the_result_function() );
   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::These_are( ':' ) );
   Tk::Help::insert_help( $dialog, [ 'bold', 'NAND' ], LogicBuddy::Singular::Use_NAND_and_NOT( '.' ) );
   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::NAND_NOT( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'bold', 'AND OR' ],
      LogicBuddy::Singular::Use_AND_9_OR_and( ', ' ) . " ^ " . LogicBuddy::Singular::for_NOT( '.' )
   );
   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::AND_OR( '.' ) );

   return;

}; # I_describe_result_function

sub I_find_file {
   my ( $name ) = @_;

   my $local_directory = File::Basename::dirname( $0 );
   my @path = ( $^O eq 'MSWin32' )  ?  (split /;/, $ENV{PATH})  :  (split /:/, $ENV{PATH});
   foreach my $dir ( $local_directory, Cwd::cwd(), @path, @INC ) {
      return "$dir/$name"                if -e "$dir/$name";
      return "$dir/etc/$name"            if -e "$dir/etc/$name";
      return "$dir/LogicBuddy/$name"     if -e "$dir/LogicBuddy/$name";
      return "$dir/LogicBuddy/etc/$name" if -e "$dir/LogicBuddy/etc/$name";
   };

   return $name;

}; # I_find_file

sub I_insert_equivalent_reserved_words {
   my ( $dialog ) = @_;

   Tk::Help::insert_help( $dialog, EMPTY, [ caption => LogicBuddy::Plural::Equivalent_reserved_word() ] );

   my @length = ();
   foreach my $entry ( LogicBuddy::Logic::Text::equivalent_symbols() ) {
      my $index = 0;
      foreach my $word ( @$entry ) {
         if ( $length[$index] ) {
            $length[$index] = length $word if $length[$index] < length $word;
         } else {
            $length[$index] = length $word;
         };
         $index++;
      };
   };

   my $symbols = EMPTY;
   foreach my $entry ( LogicBuddy::Logic::Text::equivalent_symbols() ) {
      $symbols .= "\n" if $symbols;
      my $index = 0;
      foreach my $word ( @$entry ) {
         $symbols .= $word . '  ' . ( ' ' x ( $length[$index] - length $word ) );
         $index++;
      };
   };

   Tk::Help::insert_help( $dialog, EMPTY, [ 'fixed', $symbols ] );
   return;

}; # I_insert_equivalent_reserved_words

sub I_insert_syntax_description{
   my ( $dialog ) = @_;

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Plural::Syntax_of_logical_function() ] );

   Tk::Help::insert_help( $dialog, [ 'image', 's_file' ],
      [ sub { help_syntax_example( $dialog ) }, LogicBuddy::Plural::Example_of_logical_function(), 'help_syntax_example' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_syntax_rules( $dialog ) }, LogicBuddy::Plural::Rule_of_logical_function(), 'help_syntax_rules' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_blue' ],
      [ sub { help_syntax( $dialog ) },
        LogicBuddy::Plural::Syntax_of_logical_function( ' ( ' . LogicBuddy::Singular::Current_language() .' )' ),
        'help_syntax'
      ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_blue' ],
      [ sub { help_full_syntax( $dialog ) }, LogicBuddy::Singular::Full_syntax_of_logical_function(), 'help_full_syntax' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_reserved_words( $dialog ) }, LogicBuddy::Plural::Reserved_word(), 'help_reserved_words' ]
   );

   return;

}; # I_insert_syntax_description

################################################################################
# exported subprograms
################################################################################

sub main_help {
   my ( $parent ) = @_;

   my $dialog = Tk::Help::help_dialog( $parent,                                       # $parent
                                       "LogicBuddy",                                  # $title
                                       'LogicBuddy_Main.png',                         # $image
                                       undef,                                         # $legend
                                       undef, undef,                                  # $image_width, $image_height
                                       'LogicBuddy',                                  # $icon_image_name
                                       'main_help'                                    # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY,
      "LogicBuddy " . LogicBuddy::Singular::Version() . " $LogicBuddy::Version " .
                      LogicBuddy::Singular::Release() . " $LogicBuddy::Release_type " .
                      $LogicBuddy::C_Release
   );
   Tk::Help::insert_help( $dialog, EMPTY, $C_primary_license );

   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_license( $dialog ) }, LogicBuddy::Plural::License_of_LogicBuddy(), 'help_license' ]
   );

   Tk::Help::insert_help( $dialog, EMPTY, [ caption => "LogicBuddy " . LogicBuddy::Singular::minimizes_logical_functions( '.' ) ] );

   Tk::Help::insert_help( $dialog, [ image => 'balloon' ], LogicBuddy::Singular::Balloon_Help() );
   Tk::Help::insert_help( $dialog, [ 'image', 'help' ],
      [ sub { help_dialog( $dialog ) }, LogicBuddy::Singular::Help_for_help( ': ' ) . LogicBuddy::Singular::Click(), 'help_dialog' ]
   );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Singular::Main_control_buttons() ] );
   Tk::Help::insert_help( $dialog, [ 'image', 'exit' ],  LogicBuddy::Singular::Close_LogicBuddy( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'image', 'balloon' ],  LogicBuddy::Singular::Balloon_Help_Full() );
   Tk::Help::insert_help( $dialog, [ 'image', 'flag_en' ],  LogicBuddy::Singular::Select_the_language( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'image', 'help' ],  LogicBuddy::Singular::Help_Dialog( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'image', 'viewtext' ],
      [ sub { messages_help( $dialog ) }, LogicBuddy::Plural::View_message( '.' ), 'messages_help' ]
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'NAND' ], LogicBuddy::Singular::View_Solution_Dialog( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'image', 'C_edit' ],
      [ sub { editor_help( $dialog ) }, LogicBuddy::Singular::Enter_and_change_logical_function( '.' ), 'editor_help' ]
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'editcopy' ],
      [ sub { editor_help( $dialog ) }, LogicBuddy::Singular::Copy_logical_function_to_editor( '.' ), 'editor_help' ]
   );
   Tk::Help::insert_help( $dialog, [ 'bold', 'NAND' ],
      [ sub { help_select_logic( $dialog ) }, LogicBuddy::Singular::Select_logic( '.' ), 'help_select_logic' ]
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'table' ],
      [ sub { help_define_new_table( $dialog ) }, LogicBuddy::Singular::Define_new_table( '.' ), 'help_define_new_table' ]
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'b_work' ], LogicBuddy::Singular::Solve_logical_function( '.' ) );

   I_insert_state_icons( $dialog );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Plural::Sub_window() ] );
   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_last_solution_window( $dialog ) }, LogicBuddy::Singular::Last_solution_window(), 'help_last_solution_window' ]
   );
   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_truth_table_window( $dialog ) }, LogicBuddy::Singular::Truth_table(), 'help_truth_table_window' ]
   );

   return $dialog;

}; # main_help

sub help_license {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                     # $parent
                                       LogicBuddy::Plural::License_of_LogicBuddy(),  # $title
                                       undef,        ,                               # $image
                                       undef,                                        # $legend
                                       undef, undef,                                 # $image_width, $image_height
                                       'LogicBuddy',                                 # $icon_image_name
                                       'help_license'                                # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::LogicBuddy_license() );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::The_packaged_version_of_LogicBuddy_includes_9( 'espresso', '.' ) );
   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_espresso( $dialog ) }, 'espresso', 'help_espresso' ]
   );

#  if ( $^O eq 'MSWin32' ) {
     Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Singular::On_Windows( ':' ) ] );
     Tk::Help::insert_help( $dialog, EMPTY, 
        LogicBuddy::Singular::The_packaged_version_of_LogicBuddy_includes_9( 'Strawberry Perl', '.' )
     );
     Tk::Help::insert_help( $dialog, EMPTY,
        [ sub { help_strawberry_perl( $dialog ) }, 'Strawberry Perl', 'help_strawberry_perl' ]
     );
#  };

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Plural::License() ] );

   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_artistic_license( $dialog ); }, 'The Artistic License 2.0', 'help_artistic_license' ]
   );
   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_gnu_lesser_license( $dialog ); }, 'GNU Lesser General Public License', 'help_gnu_lesser_license' ]
   );

   return $dialog;

}; # help_license

sub help_artistic_license {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                     # $parent
                                       'The Artistic License 2.0',                   # $title
                                       undef,        ,                               # $image
                                       undef,                                        # $legend
                                       undef, undef,                                 # $image_width, $image_height
                                       'LogicBuddy',                                 # $icon_image_name
                                       'help_artistic_license'                       # $label
                                     );

   Tk::Help::insert_documentation( EMPTY, $C_primary_license );
   Tk::Help::insert_documentation( EMPTY, [ html => $C_artistic, "../$C_artistic" ] );

   my $license = I_find_file( $C_artistic );
   if ( -e "$license" ) {
      if ( open my $LICENSE, '<', "$license" ) {
         my @lines = <$LICENSE>;
         Tk::Help::insert_Tk_help( $dialog, EMPTY, join '', @lines );
         close $LICENSE;
         return $dialog;
      };
   };

   Tk::Help::insert_Tk_help( $dialog, EMPTY, $C_artistic_missing );

   return $dialog;

}; # help_artistic_license

sub help_gnu_lesser_license {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                     # $parent
                                       'GNU Lesser General Public License',          # $title
                                       undef,        ,                               # $image
                                       undef,                                        # $legend
                                       undef, undef,                                 # $image_width, $image_height
                                       'LogicBuddy',                                 # $icon_image_name
                                       'help_gnu_lesser_license'                     # $label
                                     );


   Tk::Help::insert_documentation( EMPTY, $C_primary_license );
   Tk::Help::insert_documentation( EMPTY, [ html => $C_GNU, "../$C_GNU" ] );

   my $license = I_find_file( $C_GNU );
   if ( -e "$license" ) {
      if ( open my $LICENSE, '<', "$license" ) {
         my @lines = <$LICENSE>;
         Tk::Help::insert_Tk_help( $dialog, EMPTY, join '', @lines );
         close $LICENSE;
         return $dialog;
      };
   };

   Tk::Help::insert_Tk_help( $dialog, EMPTY, $C_GNU_missing );

   return $dialog;

}; # help_gnu_lesser_license

sub help_espresso {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                     # $parent
                                       'espresso',                                   # $title
                                       undef,                                        # $image
                                       undef,                                        # $legend
                                       undef, undef,                                 # $image_width, $image_height
                                       'LogicBuddy',                                 # $icon_image_name
                                       'help_espresso'                               # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::espresso_software() );
   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::The_packaged_version_of_LogicBuddy_includes_9( 'espresso', '.' ) );

   Tk::Help::insert_help( $dialog, EMPTY, [ caption => LogicBuddy::Singular::Download() ] );
   my $espresso_url = 'http://www.ecs.umass.edu/ece/labs/vlsicad/ece667/links/espresso.html';
   Tk::Help::insert_help( $dialog, [ image => 'C_download' ], [ html => $espresso_url, $espresso_url ],
   );

   return $dialog;

}; # help_espresso

sub help_strawberry_perl {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                     # $parent
                                       'Strawberry Perl',                            # $title
                                       undef,                                        # $image
                                       undef,                                        # $legend
                                       undef, undef,                                 # $image_width, $image_height
                                       'LogicBuddy',                                 # $icon_image_name
                                       'help_strawberry_perl'                        # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::Strawberry_perl() );
   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::The_packaged_version_of_LogicBuddy_includes_9( 'Strawberry Perl', '.' ) );

   Tk::Help::insert_help( $dialog, EMPTY, [ caption => LogicBuddy::Singular::Download() ] );
   my $strawberryperl_url = 'http://strawberryperl.com/';
   Tk::Help::insert_help( $dialog, [ image => 'C_download' ] , [ html => $strawberryperl_url, $strawberryperl_url ] );

   return $dialog;

}; # help_strawberry_perl

sub help_dialog {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                     # $parent
                                       LogicBuddy::Singular::Help_for_help(),        # $title
                                       undef,                                        # $image
                                       undef,                                        # $legend
                                       undef, undef,                                 # $image_width, $image_height
                                       'LogicBuddy',                                 # $icon_image_name
                                       'help_dialog'                                 # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::Description_help() );
   Tk::Help::insert_help( $dialog, [ 'image', 'help' ],
      [ sub { help_dialog( $dialog ) }, LogicBuddy::Singular::Help_for_help(), 'help_dialog' ]
   );

   return $dialog;

}; # help_dialog

sub help_last_solution_window {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                     # $parent
                                       LogicBuddy::Singular::Last_solution_window(), # $title
                                       'Solution.png',                               # $image
                                       undef,                                        # $legend
                                       undef, undef,                                 # $image_width, $image_height
                                       'LogicBuddy',                                 # $icon_image_name
                                       'help_last_solution_window'                   # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::Description_solution_window() );
   Tk::Help::insert_help( $dialog, [ 'image', 'NAND' ], LogicBuddy::Singular::View_Solution_Dialog( '.' ) );

   I_describe_result_function( $dialog );

   return $dialog;

}; # help_last_solution_window

sub help_truth_table_window {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                     # $parent
                                       LogicBuddy::Singular::Truth_table(),          # $title
                                       'Truth_table.png',                            # $image
                                       undef,                                        # $legend
                                       undef, undef,                                 # $image_width, $image_height
                                       'LogicBuddy',                                 # $icon_image_name
                                       'help_truth_table_window'                     # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::Description_truth_table() );
   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ], LogicBuddy::Singular::Description_truth_table_1() );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ], LogicBuddy::Singular::Description_truth_table_2() );
   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_change_multiple( $dialog ) },
        LogicBuddy::Singular::Change_multiple_table_entries( '.' ),
        'help_change_multiple'
      ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ], LogicBuddy::Singular::Description_truth_table_3() );
   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ], LogicBuddy::Singular::Description_truth_table_4() );
   Tk::Help::insert_help( $dialog, [ 'image', 'LogicBuddy' ],
      [ sub { help_change_multiple( $dialog ) },
        LogicBuddy::Singular::Change_multiple_table_entries(),
        'help_change_multiple'
      ]
   );
   Tk::Help::insert_help( $dialog, [ 'bold', '012' ],
      [ sub { help_change_multiple( $dialog ) },
        LogicBuddy::Singular::Row_number_of_table() . "\n" .
        LogicBuddy::Singular::Change_multiple_table_entries() . ' ' .
        LogicBuddy::Singular::using_values_of_current_row(),
        'help_change_multiple'
      ]
   );
   Tk::Help::insert_help( $dialog, [ 'bold', 'a' ],
      [ sub { help_change_variable_name( $dialog ) },
        LogicBuddy::Singular::Change_input_variable_name(),
        'help_change_variable_name'
      ]
   );
   Tk::Help::insert_help( $dialog, [ 'bold', 'x1' ],
      [ sub { help_change_variable_name( $dialog ) },
        LogicBuddy::Singular::Change_output_variable_name(),
        'help_change_variable_name'
      ]
   );
   Tk::Help::insert_help( $dialog, [ 'bold', '0' ], LogicBuddy::Singular::Logical_value( ' 0.' ) );
   Tk::Help::insert_help( $dialog, [ 'bold', '1' ], LogicBuddy::Singular::Logical_value( ' 1.' ) );
   Tk::Help::insert_help( $dialog, [ 'bold', 'd' ], LogicBuddy::Singular::Logical_value_doesnt_care( '.' ) );

   Tk::Help::insert_help( $dialog, EMPTY, [ caption => LogicBuddy::Singular::Change_multiple_table_entries() ] );
   Tk::Help::insert_help( $dialog, [ image => 's_change' ], LogicBuddy::Singular::Description_multiple_change() );
   Tk::Help::insert_help( $dialog, [ red   => '01'       ], LogicBuddy::Singular::Multiple_input_truth_table() );
   Tk::Help::insert_help( $dialog, [ red   => '01d'      ], LogicBuddy::Singular::Multiple_function_truth_table() );

   return $dialog;

}; # help_truth_table_window

sub help_change_variable_name {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                           # $parent
                                       LogicBuddy::Singular::Change_variable_name(),       # title
                                       'EingabeNamen.png',                                 # $image
                                       undef,                                              # $legend
                                       undef, undef,                                       # $image_width, $image_height
                                       'LogicBuddy',                                       # $icon_image_name
                                       'help_change_variable_name'                         # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::Enter_name() . "\n" . LogicBuddy::Singular::NAME() );
   Tk::Help::insert_help( $dialog, [ 'image', 'cancel' ],
      LogicBuddy::Singular::Cancel_dialog( ', ' ) . LogicBuddy::Singular::no_change( '.' )
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'ok' ], LogicBuddy::Singular::Accept_change( '.' ) );

   return $dialog;

}; # help_change_variable_name

sub help_select_logic {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Singular::Select_logic(),                # title
                                       undef,                                               # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_select_logic'                                  # $label
                                     );

   I_describe_result_function( $dialog );

   return $dialog;

}; # help_select_logic

sub help_define_new_table {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Singular::Define_new_table(),            # title
                                       'Tabelle_definieren.png',                            # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_define_new_table'                              # $label
                                     );

   Tk::Help::insert_help( $dialog, [ 'image', 'prev' ], LogicBuddy::Plural::decrement() );
   Tk::Help::insert_help( $dialog, [ 'image', 'next' ], LogicBuddy::Plural::increment() );
   Tk::Help::insert_help( $dialog, [ 'image', 'cancel' ], LogicBuddy::Plural::Cancel() );
   Tk::Help::insert_help( $dialog, [ 'image', 'ok' ], LogicBuddy::Plural::Ok() );

   return $dialog;

}; # help_define_new_table

sub help_change_multiple {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                               # $parent
                                       LogicBuddy::Singular::Change_multiple_table_entries(),  # title
                                       'Tabelle_mehrfach_aendern.png',                         # $image
                                       undef,                                                  # $legend
                                       undef, undef,                                           # $image_width, $image_height
                                       'LogicBuddy',                                           # $icon_image_name
                                       'help_change_multiple'                                  # $label
                                     );

   Tk::Help::insert_help( $dialog, [ image => 's_change' ], LogicBuddy::Singular::Description_multiple_change() );
   Tk::Help::insert_help( $dialog, [ image => 's_change' ], LogicBuddy::Singular::Multiple_change_upper_row() );
   Tk::Help::insert_help( $dialog, [ bold  => '?'        ], LogicBuddy::Singular::Multiple_change_lower_row() );
   Tk::Help::insert_help( $dialog, [ bold  => '1'        ], LogicBuddy::Singular::Multiple_change_toggle() );
   Tk::Help::insert_help( $dialog, [ 'bold', '0' ], LogicBuddy::Singular::Logical_value( ' 0.' ) );
   Tk::Help::insert_help( $dialog, [ 'bold', '1' ], LogicBuddy::Singular::Logical_value( ' 1.' ) );
   Tk::Help::insert_help( $dialog, [ 'bold', 'A' ], LogicBuddy::Singular::Logical_value_0_and_1( '.') );
   Tk::Help::insert_help( $dialog, [ 'bold', 'd' ], LogicBuddy::Singular::Logical_value_dosent_care( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'bold', 'X' ], LogicBuddy::Singular::Logical_value_not_changed( '.' ) );
   Tk::Help::insert_help( $dialog, [ image => 'ok'       ], LogicBuddy::Plural::Apply_change( '.' ) );
   Tk::Help::insert_help( $dialog, [ image => 'cancel'   ], LogicBuddy::Plural::Reject_change( '.' ) );

   Tk::Help::insert_help( $dialog, EMPTY, [ caption => LogicBuddy::Singular::Truth_table() ] );
   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_truth_table_window( $dialog ) }, LogicBuddy::Singular::Truth_table(), 'help_truth_table_window' ]
   );
   Tk::Help::insert_help( $dialog, [ red   => '01'       ], LogicBuddy::Singular::Multiple_input_truth_table() );
   Tk::Help::insert_help( $dialog, [ red   => '01d'      ], LogicBuddy::Singular::Multiple_function_truth_table() );

   return $dialog;

}; # help_change_multiple

sub messages_help {
   my ( $parent ) = @_;

   my $dialog = Tk::Help::help_dialog( $parent,                                             # $parent
                                       LogicBuddy::Plural::Message(),                       # $title
                                       'Messages_Window.png',                               # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'messages_help'                                      # $label
                                     );

   Tk::Help::insert_help( $dialog, [ image => 'cancel' ], LogicBuddy::Plural::Hide_message( '.' ) );
   Tk::Help::insert_help( $dialog, [ image => 'clear' ], LogicBuddy::Plural::Clear_message( '.' ) );

   return $dialog;

}; # messages_help

sub editor_help {
   my ( $parent ) = @_;

   my $dialog = Tk::Help::help_dialog( $parent,                                                 # $parent
                                       "LogicBuddy Editor",                                     # $title
                                       'LogicBuddy_Editor.png',                                 # $image
                                       undef,                                                   # $legend
                                       undef, undef,                                            # $image_width, $image_height
                                       'LogicBuddy',                                            # $icon_image_name
                                       'editor_help'                                            # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::Enter_and_change_logical_function( '.' ) );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Singular::Main_control_buttons() ] );
   Tk::Help::insert_help( $dialog, [ 'image', 'cancel' ],  LogicBuddy::Singular::Close_edit_dialog( '. ') .
                                                         LogicBuddy::Plural::The_content_of_the_dialog_is_retained( '.' )
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'help' ],  LogicBuddy::Singular::Help_Dialog( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'image', 'viewtext' ],
      [ sub { messages_help( $dialog ) }, LogicBuddy::Plural::View_message( '.' ), 'messages_help' ]
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'b_work' ],  LogicBuddy::Singular::Check_and_calculate_logical_function() );
   Tk::Help::insert_help( $dialog, [ 'image', 'clear' ], LogicBuddy::Singular::Clear_logical_function( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'image', 'next' ],  LogicBuddy::Singular::Restore_logical_function_forwards( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'image', 'prev' ],  LogicBuddy::Singular::Restore_logical_function_backwards( '.' ) );

   I_insert_state_icons( $dialog );
   Tk::Help::insert_help( $dialog, [ 'bold', LogicBuddy::Singular::Status( ':' ) ], LogicBuddy::Singular::Describes_Function_Matches() );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Singular::Automatic_error_correction() ] );
   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_error_correction( $dialog ) }, LogicBuddy::Singular::Automatic_error_correction(), 'help_error_correction' ]
   );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Plural::Sub_window() ] );
   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_editor_window( $dialog ) }, LogicBuddy::Singular::Editor_window(), 'help_editor_window' ]
   );
   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_editor_error_window( $dialog ) }, LogicBuddy::Plural::Error_message(), 'help_editor_error_window' ]
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'viewtext' ], 
      [ sub { messages_help( $dialog ) },
        LogicBuddy::Plural::The_error_messages_are_also_contained_in_the_message_window( '.' ),
        'messages_help'
      ]
   );

   I_insert_syntax_description( $dialog );

   return $dialog;

}; # editor_help

sub help_status {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Singular::Status(),                      # title
                                       undef,                                               # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_status'                                        # $label
                                     );

   I_insert_state_icons( $dialog );

   return $dialog;

}; # help_status

sub help_editor_window {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Singular::Editor_window(),               # title
                                       'Editor_Window.png',                                 # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_editor_window'                                 # $label
                                     );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ], LogicBuddy::Singular::Enter_here_the_logical_function( '.' ) );
   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ], LogicBuddy::Singular::Editor_Function_Description() );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Plural::Special_key() ] );

   Tk::Help::insert_help( $dialog, [ 'bold', 'F1' ], LogicBuddy::Singular::This_help() );
   Tk::Help::insert_help( $dialog, [ 'bold', 'F2' ],
      [ sub { help_syntax_rules( $dialog ) }, 
        LogicBuddy::Plural::Rule_of_logical_function( ' ( ' . LogicBuddy::Singular::Current_language() .' )'),
        'help_syntax_rules'
      ]
   );
   Tk::Help::insert_help( $dialog, [ 'bold', 'F3' ],
      [ sub { help_syntax( $dialog ) },
        LogicBuddy::Plural::Syntax_of_logical_function( ' ( ' . LogicBuddy::Singular::Current_language() .' )' ),
        'help_syntax'
      ]
   );
   Tk::Help::insert_help( $dialog, [ 'bold', 'F4' ],
      [ sub { help_syntax_example( $dialog ) },
        LogicBuddy::Plural::Example_of_logical_function( ' ( ' . LogicBuddy::Singular::Current_language() .' )'),
        'help_syntax_example'
      ]
   );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Singular::Syntax_highlighting() ] );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'forestgreen', LogicBuddy::Plural::Variable() ] );
   Tk::Help::insert_help( $dialog, EMPTY, [ 'mediumblue', LogicBuddy::Plural::Reserved_word() ] );
   Tk::Help::insert_help( $dialog, EMPTY, [ 'slategrey', LogicBuddy::Plural::Comment() ] );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'caption', LogicBuddy::Singular::Automatic_error_correction() ] );
   Tk::Help::insert_help( $dialog, EMPTY,
      [ sub { help_error_correction( $dialog ) }, LogicBuddy::Singular::Automatic_error_correction(), 'help_error_correction' ]
   );

   I_insert_syntax_description( $dialog );

   return $dialog;

}; # help_editor_window

sub help_error_correction {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Singular::Automatic_error_correction(),  # title
                                       'Error_Correction.png',                              # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_error_correction'                              # $label
                                     );

   
   Tk::Help::insert_help( $dialog, [ 'image', 'b_work' ], LogicBuddy::Singular::Check_Function( '.' ) );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::AUTOMATIC_ERROR_CORRECTION() );

   Tk::Help::insert_help( $dialog, [ 'bold', 'F5' ], LogicBuddy::Singular::accept_correction() );
   Tk::Help::insert_help( $dialog, [ 'image', 'prev' ],
      LogicBuddy::Plural::Discard_correction( ".\n" ) .
      LogicBuddy::Singular::Restore_logical_function_backwards( '.' )
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'b_work' ],
      LogicBuddy::Plural::Accept_correction( ".\n" ) .
      LogicBuddy::Singular::Check_and_calculate_logical_function( '.' )
   );
   Tk::Help::insert_help( $dialog, [ 'image', 'clear' ], LogicBuddy::Singular::Clear_logical_function( '.' ) );

   return $dialog;

}; # help_error_correction

sub help_syntax {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Plural::Syntax_of_logical_function(),    # title
                                       undef,                                               # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_syntax'                                        # $label
                                     );

   Tk::Help::insert_help( $dialog, [ 'image', 's_file' ],
      [ sub { help_syntax_example( $dialog ) }, LogicBuddy::Plural::Example_of_logical_function(), 'help_syntax_example' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_syntax_rules( $dialog ) }, LogicBuddy::Plural::Rule_of_logical_function(), 'help_syntax_rules' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_blue' ],
      [ sub { help_full_syntax( $dialog ) }, LogicBuddy::Singular::Full_syntax_of_logical_function(), 'help_full_syntax' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_reserved_words( $dialog ) }, LogicBuddy::Plural::Reserved_word(), 'help_reserved_words' ]
   );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'fixed', LogicBuddy::Singular::syntax_description() ] );

}; # help_syntax

sub help_full_syntax {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                                 # $parent
                                       LogicBuddy::Singular::Full_syntax_of_logical_function(),  # title
                                       undef,                                                    # $image
                                       undef,                                                    # $legend
                                       undef, undef,                                             # $image_width, $image_height
                                       'LogicBuddy',                                             # $icon_image_name
                                       'help_full_syntax'                                        # $label
                                     );

   Tk::Help::insert_help( $dialog, [ 'image', 's_file' ],
      [ sub { help_syntax_example( $dialog ) }, LogicBuddy::Plural::Example_of_logical_function(), 'help_syntax_example' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_syntax_rules( $dialog ) }, LogicBuddy::Plural::Rule_of_logical_function(), 'help_syntax_rules' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_blue' ],
      [ sub { help_syntax( $dialog ) },
        LogicBuddy::Plural::Syntax_of_logical_function( ' ( ' . LogicBuddy::Singular::Current_language() .' )' ),
        'help_syntax'
      ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_reserved_words( $dialog ) }, LogicBuddy::Plural::Reserved_word(), 'help_reserved_words' ]
   );

   Tk::Help::insert_help( $dialog, EMPTY, [ 'fixed', LogicBuddy::Singular::full_syntax_description() ] );

}; # help_full_syntax

sub help_syntax_rules {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Plural::Rule_of_logical_function(),      # title
                                       undef,                                               # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_syntax_rules'                                  # $label
                                     );

   Tk::Help::insert_help( $dialog, [ 'image', 's_file' ],
      [ sub { help_syntax_example( $dialog ) }, LogicBuddy::Plural::Example_of_logical_function(), 'help_syntax_example' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_blue' ],
      [ sub { help_syntax( $dialog ) },
        LogicBuddy::Plural::Syntax_of_logical_function( ' ( ' . LogicBuddy::Singular::Current_language() .' )' ),
        'help_syntax'
      ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_blue' ],
      [ sub { help_full_syntax( $dialog ) }, LogicBuddy::Singular::Full_syntax_of_logical_function(), 'help_full_syntax' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_reserved_words( $dialog ) }, LogicBuddy::Plural::Reserved_word(), 'help_reserved_words' ]
   );

   foreach my $rule ( LogicBuddy::Logic::Text::rules( LogicBuddy::Singular::current_language() ) ) {
      Tk::Help::insert_help( $dialog, [ image => 't_green' ], $rule );
   };

#  I_insert_equivalent_reserved_words( $dialog );

   return $dialog;

}; # help_syntax_rules

sub help_syntax_example {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Plural::Example_of_logical_function(),   # title
                                       undef,                                               # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_syntax_example'                                # $label
                                     );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_syntax_rules( $dialog ) }, LogicBuddy::Plural::Rule_of_logical_function(), 'help_syntax_rules' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_blue' ],
      [ sub { help_syntax( $dialog ) },
        LogicBuddy::Plural::Syntax_of_logical_function( ' ( ' . LogicBuddy::Singular::Current_language() .' )' ),
        'help_syntax'
      ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_blue' ],
      [ sub { help_full_syntax( $dialog ) }, LogicBuddy::Singular::Full_syntax_of_logical_function(), 'help_full_syntax' ]
   );

   Tk::Help::insert_help( $dialog, [ 'image', 't_green' ],
      [ sub { help_reserved_words( $dialog ) }, LogicBuddy::Plural::Reserved_word(), 'help_reserved_words' ]
   );

   foreach my $example ( LogicBuddy::Logic::Text::examples( LogicBuddy::Singular::current_language() ) ) {
      Tk::Help::insert_help( $dialog, [ image => 's_file' ], [ fixed => $example ] );
   };

   return $dialog;

}; # help_syntax_example

sub help_reserved_words {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Plural::Reserved_word(),                 # title
                                       undef,                                               # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_reserved_words'                                # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, [ fixed => (join "\n", LogicBuddy::Logic::reserved_words() ) ] );

   I_insert_equivalent_reserved_words( $dialog );

   return $dialog;

}; # help_reserved_words

sub help_editor_error_window {
   my ( $the_map ) = @_;

   my $dialog = Tk::Help::help_dialog( $the_map,                                            # $parent
                                       LogicBuddy::Plural::Error_message(),                 # title
                                       'Editor_Error_Message.png',                          # $image
                                       undef,                                               # $legend
                                       undef, undef,                                        # $image_width, $image_height
                                       'LogicBuddy',                                        # $icon_image_name
                                       'help_editor_error_window'                           # $label
                                     );

   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::Displays_the_last_error_message_when_checking_the_logical_function( '.' ) );
   Tk::Help::insert_help( $dialog, EMPTY, LogicBuddy::Singular::Copy_and_paste( '.' ) );

   Tk::Help::insert_help( $dialog, [ 'image', 'viewtext' ], 
      [ sub { messages_help( $dialog ) },
        LogicBuddy::Plural::The_error_messages_are_also_contained_in_the_message_window( '.' ),
        'messages_help'
      ]
   );

   return $dialog;

}; # help_editor_error_window

################################################################################
# end LogicBuddy::Tk::Help
################################################################################

1;

