=pod

Copyright Heiner Kuhlmann : hkuhlmann@dr-k.de Praktische_Elektronik.dr-k.de
The software is licensed under the GNU Library General Public License.

=head1 NAME

LogicBuddy::Utilities - module

=head1 SYNOPSIS

   use LogicBuddy::Utilities;

   @bits = LogicBuddy::Utilities::bits_of( $integer_number, $size );

=head1 ABSTRACT

LogicBuddy::Utilities provides some utilities.

=head1 DESCRIPTION

=head2 EXPORTED SUBPROGRAMS

=head3 bits_of

   @bits = LogicBuddy::Utilities::bits_of( $integer_number, $size );

returns an ordered list of $size bits of the $integer_number;

Raised exceptions: none

=head1 AUTHOR2

Heiner Kuhlmann, E<lt>hkuhlmann@dr-k.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright Heiner Kuhlmann : hkuhlmann@dr-k.de Praktische_Elektronik.dr-k.de
The software is licensed under the GNU Library General Public License.

=cut

use strict;

use Carp;

package LogicBuddy::Utilities;

use English '-no_match_vars';

################################################################################
# constants
################################################################################

use constant FALSE => 0;
use constant TRUE  => 1;
use constant EMPTY => '';

use Readonly;

################################################################################
# LogicBuddy::Utilities
################################################################################

################################################################################
# internal subprograms
################################################################################

################################################################################
# exported subprograms
################################################################################

sub bits_of ($$) {
   my ( $number, $size ) = @_;

   Carp::croak( "Illegal size $size < 1\n" ) if $size < 1;

   my @bits = ();
   foreach ( 0 .. $size - 1 ) {
      unshift @bits, $number % 2;
      $number = $number / 2;
   };

   return @bits;

}; # bits_of

################################################################################
# end LogicBuddy::Utilities
################################################################################

1;

